require "shortnr/version"
require 'digest'
require 'json'

class Shortnr
  def initialize(store:)
    validate_store!(store)
    @store = store
  end

  # Shortens a set of params to a md5 hash
  # and stores it in the store
  #
  # @param params [Hash] Params to be shortened
  # @returns [String] Shortened string
  def shorten(params)
    raise ArgumentError.new("Expected params to be a hash or array") unless params.is_a?(Hash) or params.is_a?(Array)

    shortened = Digest::MD5.hexdigest(params.to_json)
    @store.set(key_for(shortened), params.to_json)
    shortened
  end

  # Finds params for a shortened URL
  #
  # @param shortend [String] Shortened param
  # @returns [Hash] Returns a Hash or nil
  def find(shortened)
    raise ArgumentError.new("Expected shortened to be a string") unless shortened.is_a?(String)

    maybe_result = @store.get(key_for(shortened))
    if maybe_result && maybe_result.length > 0
      JSON.parse(maybe_result, symbolize_names: true)
    else
      nil
    end
  end

  private

  def validate_store!(store)
    raise ArgumentError.new("Store should respond to #set") unless store.respond_to?(:set)
    raise ArgumentError.new("Store should respond to #get") unless store.respond_to?(:get)
  end

  def key_for(shortened)
    "shortnr:#{shortened}"
  end
end
